#!/bin/bash

#########################################################################################################################################
#A script to search for the S2 thresholds of a modified version of the bidomain example problem located in 
#opencarp/share/tutorials/02_EP_tissue/01_basic_usage/runS1S2.py .
#This script requires the installation of Meshalyzer (https://opencarp.org/download/installation#meshalyzer) in /usr/local/meshalyzer/.
#Run the script with "bash S1S2.sh". 
#After the script has started, Meshalyzer will show the solutions, and the user will be prompted to indicate whether an AP has occurred.
#The clearest way to determine this is by selecting "Highlight" then "Time Series" in Meshalyzer. If the y values of the solution reach 
#   >0 at any time, an AP has fired.

#The current parameters are a test problem and are set to locate S2 thresholds for intervals 200, 205, 210, 215, 220 ms.
#########################################################################################################################################

#User sets initial values for a min and max guess.
S1_strength=155 #Usually the S1 threshold (for this problem, we will pretend 155 is the S1 threshold)
t_final=300 #duration of the experiment

S2_min=150000 #Should be set to at least the S1 threshold. (150,000 is closer to the currently set S1-S2 intervals)
S2_absolute_max=151000  #If unsure, set to maximum possible strength before solution blows up (for this problem, =225,000)

#We want to retain the value of S2_absolute_max, so copy to S2_max to start with
S2_max=$S2_absolute_max

#Find threshold up to a certain tolerance (typically 1 for these experiments).
tol=1 

#User sets the range of desired S1-S2 time intervals 
S1S2_interval_min=200
S1S2_interval_max=220
S1S2_interval_increment=5

#S1S2 interval is set to max to start with
S1S2_interval=$S1S2_interval_max

#Test the validity of the initial guesses (min-1 should not trigger an AP; max should).
./runS1S2.py --S1-strength "$S1_strength" --S2-strength "$(echo "$S2_min-1" | bc)" --S2-start "$S1S2_interval" --duration "$t_final" --visualize 
read -p "Testing 1 below S2_min, S2 = "$(echo "$S2_min-1" | bc)" uA/Cm^2: Was there a second action potential (y/n)? " ap
if [ "$ap" == "y" ]
then
    echo "Exiting: S2_min = $S2_min uA/Cm^2 triggers an action potential. Select a smaller current magnitude or shorter time interval."
    exit 1
fi

./runS1S2.py --S1-strength "$S1_strength" --S2-strength "$S2_max" --S2-start "$S1S2_interval" --duration "$t_final" --visualize 
read -p "Testing S2_max = $S2_max uA/Cm^2: Was there a second action potential (y/n)? " ap
if [ "$ap" == "n" ]
then
    echo "Exiting: S2_max = $S2_max uA/Cm^2 does not trigger an action potential. Select a larger current magnitude."
    exit 1
fi

#Define bisection function:
bisection () {
    local s2_max=$1
    local s2_min=$2
    bisection_result=$(echo "($s2_max - $s2_min)/2 + $s2_min" | bc)
    echo "$bisection_result"
}

#The main loop. Outer loop is for updating the time intervals...
while (( $(echo "$S1S2_interval - $S1S2_interval_min" | bc) >= 0))
do
    #Set current_guess equal to first bisection.
    current_guess=$(bisection $S2_max $S2_min)

#Inner loop is for updating the stimulus strengths within a given interval.
    #We want to bisect and test until the difference between the min and max guesses is equal to the tolerance.
    while (( $(echo "$S2_max - $S2_min" | bc) > $tol )) 
    do
        #Run the test to see whether current_guess fires an AP. For now, this is confirmed manually via meshalyzer visualization. Eventually the binary
        #file output by openCARP will be checked automatically (i.e. checking for  V >0mV)
        ./runS1S2.py --S1-strength "$S1_strength" --S2-strength "$current_guess" --S2-start "$S1S2_interval" --duration "$t_final" --visualize  
        
        #User enters whether or not it fired an AP.
        read -p "S2 strength = $current_guess uA/Cm^2, S1-S2 Interval = $S1S2_interval ms: Was there a second action potential (y/n)?" ap 
        
        if [ "$ap" == "y" ]
        then 
            S2_max=$current_guess #If yes, the current guess becomes the new max.
            current_guess=$(bisection $S2_max $S2_min) #Update current_guess using bisection with new S2_max.
        
        elif [ "$ap" == "n" ]
        then
            S2_min=$current_guess #If no, the current guess becomes the new min.
            current_guess=$(bisection $S2_max $S2_min) #Update current_guess using bisection with new S2_min.
        
        else
            echo "Error -- enter 'y' or 'n'" #Proper usage check
        fi

    done

    #At this point, S1_max triggers an AP while S1_min doesn't; therefore S1_max is the threshold. This is output to the file "S1S2points.txt"
    echo "The S2 threshold for a $S1S2_interval ms interval is $S2_max uA/Cm^2."
    {
        echo "The S2 threshold for a $S1S2_interval ms interval is $S2_max uA/Cm^2."
    } >> S1S2points.txt

    #Get ready to find next interval's threshold...

    #Decrement the interval. 
        # We choose to decrement rather than increment the intervals because shorter time intervals are generally more expensive to compute. 
        # Also, shorter intervals may require a higher stimulus strength than stability allows, so they are more likely to fail.)
    S1S2_interval=$(echo "$S1S2_interval-$S1S2_interval_increment" | bc)

    #Reset the min and max
    S2_min=$S2_max # A smaller interval will require a stronger stimulus; therefore, we narrow the stimulus range by setting the previous (longer) 
                   # interval's threshold to be the new interval's minimum stim strength.
    S2_max=$S2_absolute_max # Reset S2_max to the original max

done
