#!/usr/bin/env python

import os
import sys
NAME = 'Find_S1'
AUTHORS = 'Joyce Reimer, Kyle Klenk, Raymond Spiteri'
GUIinclude = True

from datetime import date
from carputils import settings
from carputils import tools
from carputils import mesh
from carputils.carpio import txt
import numpy as np
from numpy import array as nplist

def parser_for_s1():
    parser = tools.standard_parser()
    group  = parser.add_argument_group('Experiment for finding S1 threshold')
    group.add_argument('--duration',
                        type = float,
                        default = 20.,
                        help = 'Duration of simulation in [ms] (default: 20.)')
    group.add_argument('--S1-strength',
                        type = float,
                        default = 20.,
                        help = 'pick transmembrane current stimulus strength in [uA/cm^2] = [pA/pF] considering fixed Cm (default: 20.)')
    group.add_argument('--S1-dur',
                        type = float,
                        default = 2.,
                        help = 'pick transmembrane current stimulus duration in [ms] (default: 2.)')
    return parser

def jobID(args):
    """
    Generate name of top level output directory.
    """
    today = date.today()
    return 'simulations/output/s1_attempt_{}'.format(int(args.S1_strength))

@tools.carpexample(parser_for_s1, jobID)
def run(args, job):
        #===========================================
    # 1: Define the mesh: Units are mm
    #===========================================
    
    # Create a block with a thin z dimension
    x = 10.0 #(mm)
    y = 1.0  #(mm)
    z = 0.2  #(mm)
    geom = mesh.Block(size=(x,y,z))

    # Set fibre angle to 0, sheet angle to 0
    geom.set_fibres(0, 0, 90, 90) #endo, epi, sheet endo, sheet epicard

    # Finilaize the creation of the mesh
    meshname = mesh.generate(geom)

    #===========================================
    # 2: Define the ionic models & conductivities
    #===========================================

    # Query for element labels
    _, etags, _ = txt.read(meshname + '.elem')
    etags = np.unique(etags)

    IntraTags = etags[etags != 0].tolist()
    ExtraTags = etags.tolist().copy()

    #===========================================
    # 3: Configure the simulation
    #===========================================

    # Define the geometry of the stimulus at one end of the block
    # Units are um
    stim = ['-num_stim',             1,
            '-stimulus[0].name',    'S1',
            '-stimulus[0].stimtype', 0,
            '-stimulus[0].strength', args.S1_strength,
            '-stimulus[0].duration', args.S1_dur ]
    
    electrode = mesh.block_boundary_condition(geom, 'stimulus', 0, 'x', True)

    #===========================================
    # 4: Defining simulator options:
    #===========================================

    # Get basic command line, including solver options
    cmd = tools.carp_cmd(os.path.join('simulations/input/basic.par')) #cmd is a carp command

    cmd += ['-simID',    job.ID,
            '-meshname', meshname,
            '-dt',       25,
            '-tend',     args.duration]
    cmd += tools.gen_physics_opts(ExtraTags=ExtraTags, IntraTags=IntraTags)
    cmd += stim + electrode

    # Run simulation 
    job.carp(cmd) #cmd includes the params

run()
